# Copyright (c) 2016 Catalyst IT Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from unittest import mock

from zaqarclient.tests.queues import base
from zaqarclient.transport import response


class QueuesV2HealthUnitTest(base.QueuesTestBase):

    def test_health(self):
        expect_health = {
            'catalog_reachable': True,
            'redis': {
                'operation_status': {},
                'storage_reachable': True
            }
        }

        with mock.patch.object(self.transport, 'send',
                               autospec=True) as send_method:

            health_content = json.dumps(expect_health)
            health_resp = response.Response(None, health_content)
            send_method.side_effect = iter([health_resp])

            health = self.client.health()
            self.assertEqual(expect_health, health)


class QueuesV2HealthFunctionalTest(base.QueuesTestBase):
    def test_ping(self):
        # NOTE(flwang): If test env is not pingable, then the test should fail
        self.assertTrue(self.client.ping())

    def test_health(self):
        health = self.client.health()
        # NOTE(flwang): If everything is ok, then zaqar server will return a
        # JSON(dict).
        self.assertIsInstance(health, dict)



# Copyright 2014 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from zaqarclient.queues import client

URL = 'http://localhost:8888'


def healthy():
    # Note: credential information should be provided
    # using `conf` keyword argument if authentication
    # is enabled at server side. Please refer to
    # keystone_auth.py for more information.
    cli = client.Client(url=URL, version=2)
    return True if cli.health() else False


if __name__ == '__main__':
    healthy()



# Copyright (c) 2013 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from zaqarclient.transport import api


class FakeApi(api.Api):
    label = 'v2'

    schema = {
        'test_operation': {
            'ref': 'test/{name}',
            'method': 'GET',
            'properties': {
                'name': {'type': 'string'},
                'address': {'type': 'string'}
            },

            'additionalProperties': False,
            'required': ['name']
        }
    }



# Copyright (c) 2013 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
A `Client` is a high-level abstraction on top of Zaqar features. It
exposes the server features with an object-oriented interface, which
encourages dot notation and automatic, but lazy, resources
allocation. A `Client` allows you to control everything, from public
interfaces to admin endpoints.

To create a `Client` instance, you supply an url pointing to the
server and a version number::

    from zaqarclient.queues import client

    cli = client.Client(\'http://zaqar.example.com:8888/\', version=2)

which will load the appropriate client based on the specified
version. Optionally, you can also supply a config dictionary::

    from zaqarclient.queues import client

    cli = client.Client(\'http://zaqar.example.com:8888/\',
                        version=2, conf={})

The arguments passed to this function will be passed to the client
instances as well.

It's recommended to use `Client` instances instead of accessing the
lower level API as it has been designed to ease the interaction with
the server and it gives enough control for the most common cases.

A simple example for accessing an existing queue through a client
instance - based on the API v2 - would look like::

    from zaqarclient.queues import client

    cli = client.Client(\'http://zaqar.example.com:8888/\', version=2)
    queue = cli.queue(\'my_queue\')

Through the queue instance will be then possible to access all the
features associated with the queue itself like posting messages,
getting message and deleting messages.

As mentioned previously in this documentation, a client instance
allows you to also access admin endpoints, for example::

    from zaqarclient.queues import client

    cli = client.Client(\'http://zaqar.example.com:8888/\', version=2)
    flavor = cli.flavor(\'tasty\',
                        pool=\'my-pool-group\',
                        auto_create=True)
    flavor.delete()

`Client` uses the lower-level API to access the server, which means
anything you can do with this client instance can be done by accessing
the underlying API, although not recommended.
"""
from zaqarclient import errors
from zaqarclient.queues.v2 import client as cv2

_CLIENTS = {2: cv2.Client}


def Client(url=None, version=None, conf=None, session=None):
    # NOTE: Please don't mix use the Client object with different version at
    # the same time. Because the cache mechanism of queue's metadata will lead
    # to unexpected response value.
    # Please see zaqarclient.queues.v2.queues.Queue.metadata for more detail.
    try:
        return _CLIENTS[version](url=url, version=version, conf=conf,
                                 session=session)
    except KeyError:
        raise errors.ZaqarError('Unknown client version')



# Copyright (c) 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from zaqarclient.transport import api


class V2(api.Api):
    label = 'v2'
    schema = {
        'queue_list': {
            'ref': 'queues',
            'method': 'GET',
            'properties': {
                'marker': {'type': 'string'},
                'limit': {'type': 'integer'},
                'detailed': {'type': 'boolean'}
            }
        },
        'queue_create': {
            'ref': 'queues/{queue_name}',
            'method': 'PUT',
            'required': ['queue_name'],
            'properties': {
                'queue_name': {'type': 'string'}
            },
        },
        'queue_get': {
            'ref': 'queues/{queue_name}',
            'method': 'GET',
            'required': ['queue_name'],
            'properties': {
                'queue_name': {'type': 'string'}
            }
        },
        'queue_exists': {
            'ref': 'queues/{queue_name}',
            'method': 'HEAD',
            'required': ['queue_name'],
            'properties': {
                'queue_name': {'type': 'string'}
            }
        },
        'queue_update': {
            'ref': 'queues/{queue_name}',
            'method': 'PATCH',
            'required': ['queue_name'],
            'properties': {
                'queue_name': {'type': 'string'}
            }
        },
        'queue_delete': {
            'ref': 'queues/{queue_name}',
            'method': 'DELETE',
            'required': ['queue_name'],
            'properties': {
                'queue_name': {'type': 'string'}
            }
        },
        'queue_get_stats': {
            'ref': 'queues/{queue_name}/stats',
            'method': 'GET',
            'required': ['queue_name'],
            'properties': {
                'queue_name': {'type': 'string'}
            }
        },
        'queue_purge': {
            'ref': 'queues/{queue_name}/purge',
            'method': 'POST',
            'required': ['queue_name'],
            'properties': {
                'queue_name': {'type': 'string'}
            }
        },

        'message_list': {
            'ref': 'queues/{queue_name}/messages',
            'method': 'GET',
            'required': ['queue_name'],
            'properties': {
                'queue_name': {'type': 'string'},
                'marker': {'type': 'string'},
                'limit': {'type': 'integer'},
                'echo': {'type': 'boolean'},
                'include_claimed': {'type': 'boolean'},
            }
        },
        'message_post': {
            'ref': 'queues/{queue_name}/messages',
            'method': 'POST',
            'required': ['queue_name', 'message_id'],
            'properties': {
                'queue_name': {'type': 'string'},
                'claim_id': {'type': 'string'},
            }
        },
        'message_get': {
            'ref': 'queues/{queue_name}/messages/{message_id}',
            'method': 'GET',
            'required': ['queue_name', 'message_id'],
            'properties': {
                'queue_name': {'type': 'string'},
                'message_id': {'type': 'string'},
                'claim_id': {'type': 'string'},
            }
        },
        'message_get_many': {
            'ref': 'queues/{queue_name}/messages',
            'method': 'GET',
            'required': ['queue_name', 'ids'],
            'properties': {
                'queue_name': {'type': 'string'},
                'ids': {'type': 'string'},
                'claim_id': {'type': 'string'},
            }
        },
        'message_delete': {
            'ref': 'queues/{queue_name}/messages/{message_id}',
            'method': 'DELETE',
            'required': ['queue_name', 'message_id'],
            'properties': {
                'queue_name': {'type': 'string'},
                'message_id': {'type': 'string'},
                'claim_id': {'type': 'string'},
            }
        },
        'message_delete_many': {
            'ref': 'queues/{queue_name}/messages',
            'method': 'DELETE',
            'required': ['queue_name', 'ids'],
            'properties': {
                'queue_name': {'type': 'string'},
                'ids': {'type': 'string'},
            }
        },
        'message_pop': {
            'ref': 'queues/{queue_name}/messages',
            'method': 'DELETE',
            'required': ['queue_name', 'pop'],
            'properties': {
                'queue_name': {'type': 'string'},
                'pop': {'type': 'integer'},
            }
        },

        'pool_list': {
            'ref': 'pools',
            'method': 'GET',
            'properties': {
                'pool_name': {'type': 'string'},
                'marker': {'type': 'string'},
                'limit': {'type': 'integer'},
                'detailed': {'type': 'boolean'}
            }
        },
        'pool_create': {
            'ref': 'pools/{pool_name}',
            'method': 'PUT',
            'required': ['pool_name'],
            'properties': {
                'pool_name': {'type': 'string'},
            }
        },
        'pool_get': {
            'ref': 'pools/{pool_name}',
            'method': 'GET',
            'required': ['pool_name'],
            'properties': {
                'pool_name': {'type': 'string'},
            }
        },
        'pool_update': {
            'ref': 'pools/{pool_name}',
            'method': 'PATCH',
            'required': ['pool_name'],
            'properties': {
                'pool_name': {'type': 'string'}
            }
        },
        'pool_delete': {
            'ref': 'pools/{pool_name}',
            'method': 'DELETE',
            'required': ['pool_name'],
            'properties': {
                'pool_name': {'type': 'string'},
            }
        },

        'subscription_list': {
            'ref': 'queues/{queue_name}/subscriptions',
            'method': 'GET',
            'properties': {
                'marker': {'type': 'string'},
                'limit': {'type': 'integer'},
                'detailed': {'type': 'boolean'}
            }
        },
        'subscription_create': {
            'ref': 'queues/{queue_name}/subscriptions',
            'method': 'POST',
            'required': ['queue_name'],
            'properties': {
                'queue_name': {'type': 'string'}
            },
        },
        'subscription_get': {
            'ref': 'queues/{queue_name}/subscriptions/{subscription_id}',
            'method': 'GET',
            'required': ['queue_name', 'subscription_id'],
            'properties': {
                'queue_name': {'type': 'string'},
                'subscription_id': {'type': 'string'}
            },
        },
        'subscription_update': {
            'ref': 'queues/{queue_name}/subscriptions/{subscription_id}',
            'method': 'PATCH',
            'required': ['queue_name', 'subscription_id'],
            'properties': {
                'queue_name': {'type': 'string'},
                'subscription_id': {'type': 'string'}
            }
        },
        'subscription_delete': {
            'ref': 'queues/{queue_name}/subscriptions/{subscription_id}',
            'method': 'DELETE',
            'required': ['queue_name', 'subscription_id'],
            'properties': {
                'queue_name': {'type': 'string'},
                'subscription_id': {'type': 'string'}
            }
        },

        'flavor_list': {
            'ref': 'flavors',
            'method': 'GET',
            'properties': {
                'flavor_name': {'type': 'string'},
                'marker': {'type': 'string'},
                'limit': {'type': 'integer'},
                'detailed': {'type': 'boolean'}
            }
        },
        'flavor_create': {
            'ref': 'flavors/{flavor_name}',
            'method': 'PUT',
            'required': ['flavor_name'],
            'properties': {
                'flavor_name': {'type': 'string'},
            }
        },
        'flavor_get': {
            'ref': 'flavors/{flavor_name}',
            'method': 'GET',
            'required': ['flavor_name'],
            'properties': {
                'flavor_name': {'type': 'string'},
            }
        },
        'flavor_update': {
            'ref': 'flavors/{flavor_name}',
            'method': 'PATCH',
            'required': ['flavor_name'],
            'properties': {
                'flavor_name': {'type': 'string'}
            }
        },
        'flavor_delete': {
            'ref': 'flavors/{flavor_name}',
            'method': 'DELETE',
            'required': ['flavor_name'],
            'properties': {
                'flavor_name': {'type': 'string'},
            }
        },

        'claim_create': {
            'ref': 'queues/{queue_name}/claims',
            'method': 'POST',
            'required': ['queue_name'],
            'properties': {
                'queue_name': {'type': 'string'},
                'limit': {'type': 'integer'},
                'grace': {'type': 'integer'}
            }
        },
        'claim_get': {
            'ref': 'queues/{queue_name}/claims/{claim_id}',
            'method': 'GET',
            'required': ['queue_name', 'claim_id'],
            'properties': {
                'queue_name': {'type': 'string'},
                'claim_id': {'type': 'string'}
            }
        },
        'claim_update': {
            'ref': 'queues/{queue_name}/claims/{claim_id}',
            'method': 'PATCH',
            'required': ['queue_name', 'claim_id'],
            'properties': {
                'queue_name': {'type': 'string'},
                'claim_id': {'type': 'string'}
            }
        },
        'claim_delete': {
            'ref': 'queues/{queue_name}/claims/{claim_id}',
            'method': 'DELETE',
            'required': ['queue_name', 'claim_id'],
            'properties': {
                'queue_name': {'type': 'string'},
                'claim_id': {'type': 'string'}
            }
        },

        'signed_url_create': {
            'ref': 'queues/{queue_name}/share',
            'method': 'POST',
            'required': ['queue_name'],
            'properties': {
                'queue_name': {'type': 'string'}
            },
        },

        'ping': {
            'ref': 'ping',
            'method': 'GET',
        },

        'health': {
            'ref': 'health',
            'method': 'GET',
        },

        'homedoc': {
            'ref': '',
            'method': 'GET',
        },
    }



# Copyright (c) 2013 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

import requests as prequest
from requests.packages.urllib3 import response

from zaqarclient.tests import base
from zaqarclient.tests.transport import api
from zaqarclient.transport import http
from zaqarclient.transport import request


class TestHttpTransport(base.TestBase):

    """Tests for the HTTP transport."""

    def setUp(self):
        super().setUp()
        self.api = api.FakeApi()
        self.transport = http.HttpTransport(self.conf)

    @mock.patch.object(prequest.packages.urllib3.response.HTTPResponse,
                       'stream')
    def test_basic_send(self, mock_stream):
        params = {'name': 'Test',
                  'address': 'Outer space'}
        req = request.Request('http://example.org/',
                              operation='test_operation',
                              params=params)

        with mock.patch.object(self.transport.client, 'request',
                               autospec=True) as request_method:

            resp = prequest.Response()
            raw = response.HTTPResponse()
            resp.raw = raw
            request_method.return_value = resp

            # NOTE(flaper87): Bypass the API
            # loading step by setting the _api
            # attribute
            req._api = self.api
            self.transport.send(req)

            final_url = 'http://example.org/v2/test/Test'
            final_params = {'address': 'Outer space'}
            final_headers = {'content-type': 'application/json'}

            request_method.assert_called_with('GET', url=final_url,
                                              params=final_params,
                                              headers=final_headers,
                                              data=None,
                                              verify=True)

    @mock.patch.object(prequest.packages.urllib3.response.HTTPResponse,
                       'stream')
    def test_send_without_api(self, mock_stream):
        params = {'name': 'Test',
                  'address': 'Outer space'}
        req = request.Request('http://example.org/',
                              operation='test_operation',
                              params=params)

        with mock.patch.object(self.transport.client, 'request',
                               autospec=True) as request_method:

            resp = prequest.Response()
            raw = response.HTTPResponse()
            resp.raw = raw
            request_method.return_value = resp
            self.transport.send(req)

            final_url = 'http://example.org/'
            final_headers = {'content-type': 'application/json'}

            request_method.assert_called_with('GET', url=final_url,
                                              params=params,
                                              headers=final_headers,
                                              data=None,
                                              verify=True)

    @mock.patch.object(prequest.packages.urllib3.response.HTTPResponse,
                       'stream')
    def test_error_handling(self, mock_stream):
        params = {'name': 'Opportunity',
                  'address': 'NASA'}
        req = request.Request('http://example.org/',
                              operation='test_operation',
                              params=params)

        with mock.patch.object(self.transport.client, 'request',
                               autospec=True) as request_method:

            exception_iterator = self.transport.http_to_zaqar.items()

            for response_code, exception in exception_iterator:

                resp = prequest.Response()
                raw = response.HTTPResponse()
                resp.raw = raw
                resp.status_code = response_code
                request_method.return_value = resp
                self.assertRaises(exception, lambda: self.transport.send(req))
